"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkupManager = void 0;
const uuid_1 = require("uuid");
const utils_1 = require("../utils");
class MarkupManager extends utils_1.SafeEventEmitter {
    constructor(logger) {
        super();
        this._markups = new Map();
        this._nodes = new Map();
        this.logger = logger;
        this._namespace = (0, uuid_1.v4)();
    }
    /**
     * Set raw markup for a specific id
     */
    setMarkup(id, rawMarkup) {
        var _a, _b;
        if (typeof rawMarkup !== 'string') {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.error('[MarkupManager] rawMarkup must be a string');
            return;
        }
        this._markups.set(id, rawMarkup);
        this.emit('markupChanged', { id, rawMarkup });
        (_b = this.logger) === null || _b === void 0 ? void 0 : _b.log(`[MarkupManager] Raw markup for ID ${id} set successfully`);
    }
    /**
     * Set a node for a specific id
     */
    setNode(id, node) {
        var _a, _b;
        if (!node) {
            (_a = this.logger) === null || _a === void 0 ? void 0 : _a.error('[MarkupManager] Node must be a valid ProseMirror Node');
            return;
        }
        this._nodes.set(id, node);
        this.emit('nodeChanged', { id, node });
        (_b = this.logger) === null || _b === void 0 ? void 0 : _b.log(`[MarkupManager] Node for ID ${id} set successfully`);
    }
    /**
     * Get raw markup for a specific id
     */
    getMarkup(id) {
        var _a;
        return (_a = this._markups.get(id)) !== null && _a !== void 0 ? _a : null;
    }
    /**
     * Get a node for a specific id
     */
    getNode(id) {
        var _a;
        return (_a = this._nodes.get(id)) !== null && _a !== void 0 ? _a : null;
    }
    getNamespace() {
        return this._namespace;
    }
    /**
     * Reset the stored markups and nodes
     */
    reset() {
        var _a;
        this._markups.clear();
        this._nodes.clear();
        this.emit('reset', {});
        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.log('[MarkupManager] MarkupManager has been reset');
    }
}
exports.MarkupManager = MarkupManager;
