import MarkdownIt, { PresetName } from 'markdown-it';
import { Schema } from 'prosemirror-model';
import type { Plugin } from 'prosemirror-state';
import { ActionsManager } from './ActionsManager';
import { SchemaDynamicModifier } from './SchemaDynamicModifier';
import { MarkdownParserDynamicModifier } from './markdown/MarkdownParser';
import { MarkdownSerializerDynamicModifier } from './markdown/MarkdownSerializerDynamicModifier';
import { TransformFn } from './markdown/ProseMirrorTransformer';
import type { ActionSpec } from './types/actions';
import type { Extension, ExtensionDeps } from './types/extension';
import type { MarkViewConstructor, NodeViewConstructor } from './types/node-views';
declare type ExtensionsManagerParams = {
    extensions: Extension;
    options?: ExtensionsManagerOptions;
};
declare type ExtensionsManagerOptions = {
    mdOpts?: MarkdownIt.Options & {
        preset?: PresetName;
    };
    linkifyTlds?: string | string[];
    pmTransformers?: TransformFn[];
    dynamicModifiers?: {
        parser?: MarkdownParserDynamicModifier;
        serializer?: MarkdownSerializerDynamicModifier;
        schema?: SchemaDynamicModifier;
    };
};
export declare class ExtensionsManager {
    #private;
    static process(extensions: Extension, options: ExtensionsManagerOptions): {
        actions: ActionsManager;
        rawActions: Record<string, ActionSpec>;
        plugins: Plugin<any>[];
        nodeViews: Record<string, NodeViewConstructor>;
        markViews: Record<string, MarkViewConstructor>;
        schema: Schema<any, any>;
        textParser: import(".").Parser;
        markupParser: import(".").Parser;
        serializer: import(".").Serializer;
    };
    constructor({ extensions, options }: ExtensionsManagerParams);
    build(): {
        actions: ActionsManager;
        rawActions: Record<string, ActionSpec>;
        plugins: Plugin<any>[];
        nodeViews: Record<string, NodeViewConstructor>;
        markViews: Record<string, MarkViewConstructor>;
        schema: Schema<any, any>;
        textParser: import(".").Parser;
        markupParser: import(".").Parser;
        serializer: import(".").Serializer;
    };
    buildDeps(): ExtensionDeps;
    private processExtensions;
    private processNode;
    private processMark;
    private createParser;
    private createDeps;
    private createDerived;
}
export {};
