"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSchemaSpecs = void 0;
const prosemirror_model_1 = require("prosemirror-model");
const const_1 = require("./const");
const DEFAULT_LABEL_PLACEHOLDER = 'Checkbox';
const getSchemaSpecs = (opts, placeholder) => {
    var _a, _b;
    return ({
        [const_1.CheckboxNode.Checkbox]: {
            group: 'block checkbox',
            content: `${const_1.CheckboxNode.Input} ${const_1.CheckboxNode.Label}`,
            selectable: true,
            allowSelection: false,
            attrs: {
                [const_1.CheckboxAttr.Class]: { default: (0, const_1.b)() },
            },
            parseDOM: [
                {
                    tag: 'div.checkbox',
                    priority: 100,
                    getContent(node, schema) {
                        const input = node.querySelector('input[type=checkbox]');
                        const label = node.querySelector('label[for]');
                        const checked = (input === null || input === void 0 ? void 0 : input.checked) ? 'true' : null;
                        const text = label === null || label === void 0 ? void 0 : label.textContent;
                        return prosemirror_model_1.Fragment.from([
                            (0, const_1.checkboxInputType)(schema).create({ [const_1.CheckboxAttr.Checked]: checked }),
                            (0, const_1.checkboxLabelType)(schema).create(null, text ? schema.text(text) : null),
                        ]);
                    },
                },
                {
                    tag: 'input[type=checkbox]',
                    priority: 50,
                    getContent(node, schema) {
                        var _a, _b;
                        const id = node.id;
                        const checked = node.checked ? 'true' : null;
                        const text = (_b = (_a = node.parentNode) === null || _a === void 0 ? void 0 : _a.querySelector(`label[for=${id}]`)) === null || _b === void 0 ? void 0 : _b.textContent;
                        return prosemirror_model_1.Fragment.from([
                            (0, const_1.checkboxInputType)(schema).create({ [const_1.CheckboxAttr.Checked]: checked }),
                            (0, const_1.checkboxLabelType)(schema).create(null, text ? schema.text(text) : null),
                        ]);
                    },
                },
            ],
            toDOM(node) {
                return ['div', node.attrs, 0];
            },
            complex: 'root',
        },
        [const_1.CheckboxNode.Input]: {
            group: 'block checkbox',
            parseDOM: [],
            attrs: {
                [const_1.CheckboxAttr.Type]: { default: 'checkbox' },
                [const_1.CheckboxAttr.Id]: { default: null },
                [const_1.CheckboxAttr.Checked]: { default: null },
            },
            toDOM(node) {
                return ['div', node.attrs];
            },
            selectable: false,
            allowSelection: false,
            complex: 'leaf',
        },
        [const_1.CheckboxNode.Label]: {
            content: 'inline*',
            group: 'block checkbox',
            parseDOM: [
                {
                    tag: `span[class="${(0, const_1.b)('label')}"]`,
                    getAttrs: (node) => ({
                        [const_1.CheckboxAttr.For]: node.getAttribute(const_1.CheckboxAttr.For) || '',
                    }),
                },
                {
                    // input handled by checkbox node parse rule
                    // ignore label
                    tag: 'input[type=checkbox] ~ label[for]',
                    ignore: true,
                    consuming: true,
                },
            ],
            attrs: {
                [const_1.CheckboxAttr.For]: { default: null },
            },
            escapeText: false,
            placeholder: {
                content: (_b = (_a = placeholder === null || placeholder === void 0 ? void 0 : placeholder[const_1.CheckboxNode.Label]) !== null && _a !== void 0 ? _a : opts === null || opts === void 0 ? void 0 : opts.checkboxLabelPlaceholder) !== null && _b !== void 0 ? _b : DEFAULT_LABEL_PLACEHOLDER,
                alwaysVisible: true,
            },
            toDOM(node) {
                return ['span', Object.assign(Object.assign({}, node.attrs), { class: (0, const_1.b)('label') }), 0];
            },
            selectable: false,
            allowSelection: false,
            disableGapCursor: true,
            complex: 'leaf',
        },
    });
};
exports.getSchemaSpecs = getSchemaSpecs;
