import { ToolbarName } from '../../../modules/toolbars/constants';
import { commonmark, defaultPreset, full, yfm, zero } from '../../../modules/toolbars/presets';
import { ToolbarDataType } from '../types';
import { flattenPreset } from './flattenPreset';
const defaultPresets = {
    zero,
    commonmark,
    default: defaultPreset,
    yfm,
    full,
};
const transformItem = (type, item, id = 'unknown') => {
    var _a;
    if (!item) {
        console.warn(`Toolbar item "${id}" not found, it might not have been added to the items dictionary.`);
        return {};
    }
    const isListButton = item.view.type === ToolbarDataType.ListButton;
    const isSingleButton = item.view.type === ToolbarDataType.SingleButton;
    return Object.assign(Object.assign(Object.assign(Object.assign({ type: (_a = item.view.type) !== null && _a !== void 0 ? _a : ToolbarDataType.SingleButton, id, title: item.view.title, hint: item.view.hint, icon: item.view.icon, hotkey: item.view.hotkey }, (isSingleButton && { preview: item.view.preview })), (isListButton && { withArrow: item.view.withArrow })), (type === 'wysiwyg' && item.wysiwyg && Object.assign({}, item.wysiwyg))), (type === 'markup' && item.markup && Object.assign({}, item.markup)));
};
export const createToolbarConfig = (editorType, toolbarPreset, toolbarName) => {
    var _a;
    const preset = typeof toolbarPreset === 'string'
        ? defaultPresets[toolbarPreset] || defaultPresets.default
        : toolbarPreset;
    const orders = (_a = preset.orders[toolbarName]) !== null && _a !== void 0 ? _a : [[]];
    const { items } = preset;
    const toolbarData = orders.map((group) => group.map((action) => {
        return typeof action === 'string'
            ? transformItem(editorType, items[action], action)
            : Object.assign(Object.assign({}, transformItem(editorType, items[action.id], action.id)), { data: action.items.map((id) => transformItem(editorType, items[id], id)) });
    }));
    return toolbarData;
};
export const getToolbarsConfigs = ({ toolbarsPreset, props, preset }) => {
    var _a, _b, _c, _d;
    const wysiwygToolbarConfig = toolbarsPreset
        ? createToolbarConfig('wysiwyg', toolbarsPreset, ToolbarName.wysiwygMain)
        : (_a = props.wysiwygToolbarConfig) !== null && _a !== void 0 ? _a : createToolbarConfig('wysiwyg', preset, ToolbarName.wysiwygMain);
    const markupToolbarConfig = toolbarsPreset
        ? createToolbarConfig('markup', toolbarsPreset, ToolbarName.markupMain)
        : (_b = props.markupToolbarConfig) !== null && _b !== void 0 ? _b : createToolbarConfig('markup', preset, ToolbarName.markupMain);
    const wysiwygHiddenActionsConfig = toolbarsPreset
        ? flattenPreset(createToolbarConfig('wysiwyg', toolbarsPreset, ToolbarName.wysiwygHidden))
        : (_c = props.wysiwygHiddenActionsConfig) !== null && _c !== void 0 ? _c : flattenPreset(createToolbarConfig('wysiwyg', preset, ToolbarName.wysiwygHidden));
    const markupHiddenActionsConfig = toolbarsPreset
        ? flattenPreset(createToolbarConfig('markup', toolbarsPreset, ToolbarName.markupHidden))
        : (_d = props.markupHiddenActionsConfig) !== null && _d !== void 0 ? _d : flattenPreset(createToolbarConfig('markup', preset, ToolbarName.markupHidden));
    return {
        wysiwygToolbarConfig,
        markupToolbarConfig,
        wysiwygHiddenActionsConfig,
        markupHiddenActionsConfig,
    };
};
