import type { Match } from 'linkify-it';
import type MarkdownIt from 'markdown-it';
import type Token from 'markdown-it/lib/token';
import { Mark, Node, NodeType, Schema } from 'prosemirror-model';
import type { Parser, ParserToken } from '../types/parser';
import { TransformFn } from './ProseMirrorTransformer';
export declare type TokenAttrs = {
    [name: string]: unknown;
};
export declare class MarkdownParser implements Parser {
    schema: Schema;
    stack: Array<{
        type: NodeType;
        attrs?: TokenAttrs;
        content: Array<Node>;
    }>;
    marks: readonly Mark[];
    tokens: Record<string, ParserToken>;
    tokenizer: MarkdownIt;
    pmTransformers: TransformFn[];
    dynamicModifier: MarkdownParserDynamicModifier | null;
    constructor(schema: Schema, tokenizer: MarkdownIt, tokens: Record<string, ParserToken>, pmTransformers: TransformFn[], dynamicModifier?: MarkdownParserDynamicModifier);
    validateLink(url: string): boolean;
    normalizeLink(url: string): string;
    normalizeLinkText(url: string): string;
    matchLinks(text: string): Readonly<Match>[] | null;
    parse(text: string): Node;
    private top;
    private push;
    private getTokenAttrs;
    private getTokenSpec;
    private getTokenType;
    private handlePrimitiveToken;
    private handleMark;
    private getNodeSchema;
    private handleNode;
    private handleBlock;
    private handleToken;
    private addNode;
    private openNode;
    private closeNode;
    private addText;
    private openMark;
    private closeMark;
    private parseTokens;
}
export declare type ProcessNodeAttrs = (token: Token, attrs: TokenAttrs, allowedAttrs?: string[]) => TokenAttrs;
export declare type ProcessNode = (node: Node) => Node;
